@@ -1,6 +1,9 @@
 import base64
 import json
 import os
+import shutil
+import sys
+import tempfile
 import traceback
 from math import ceil
 from multiprocessing import Pool
@@ -8,15 +11,15 @@ from operator import itemgetter
 from pathlib import Path
 from platform import system
 from subprocess import run, PIPE, Popen
-from shutil import rmtree, copyfile
-from tempfile import mkdtemp
+from shutil import copytree, rmtree, copyfile
+from tempfile import NamedTemporaryFile, mkdtemp
 from time import sleep
 from threading import Thread
 from typing import List
 from xml.dom import minidom
 
 import requests
-import webviewb
+import webview
 
 from bcml import DEBUG, install, dev, mergers, upgrade, util
 from bcml.gamebanana import GameBananaDb
@@ -41,16 +44,19 @@ def win_or_lose(func):
 
 def start_new_instance():
     sleep(0.33)
-    Popen([util.get_python_exe(), "-m", "bcml"], cwd=str(Path().resolve()))
+    Popen(
+        [sys.executable.replace("python.exe", "pythonw.exe"), "-m", "bcml"],
+        cwd=str(Path().resolve()),
+    )
 
 
 def help_window(host: str):
-    webviewb.create_window("BCML Help", url=f"{host}/help/")
+    webview.create_window("BCML Help", url=f"{host}/help/")
 
 
 class Api:
     # pylint: disable=unused-argument,no-self-use,too-many-public-methods
-    window: webviewb.Window
+    window: webview.Window
     host: str
     gb_api: GameBananaDb
     tmp_files: List[Path]
@@ -60,18 +66,35 @@ class Api:
         self.tmp_files = []
 
     def get_ver(self, params=None):
-        return {
+        updated = util.get_settings("last_version") < VERSION
+        res = {
             "version": USER_VERSION,
-            "update": util.get_latest_bcml() > VERSION
-            and not util.get_settings("suppress_update"),
+            "update": (
+                util.get_latest_bcml() > VERSION
+                and not util.get_settings("suppress_update")
+            ),
+            "showChangelog": updated and util.get_settings("changelog"),
         }
+        if updated:
+            util.get_settings()["last_version"] = VERSION
+            util.save_settings()
+        return res
 
     @win_or_lose
     def sanity_check(self, kwargs=None):
         util.sanity_check()
 
     def get_folder(self):
-        return self.window.create_file_dialog(webviewb.FOLDER_DIALOG)[0]
+        if SYSTEM == "Windows":
+            from tkinter import filedialog
+            from tkinter import Tk
+
+            root = Tk()
+            root.attributes("-alpha", 0.0)
+            folder = filedialog.askdirectory(parent=root)
+            return folder if folder != "" else None
+        else:
+            return self.window.create_file_dialog(webview.FOLDER_DIALOG)[0]
 
     def dir_exists(self, params):
         path = Path(params["folder"])
@@ -90,6 +113,27 @@ class Api:
             return True
         return True
 
+    def drill_dir(self, params):
+        folder = Path(params["folder"])
+        if not folder.exists():
+            return params["folder"]
+        folder = folder.parent
+        if "game_dir" in params["type"]:
+            targets: List[Path] = list(folder.rglob("**/Pack/Dungeon000.pack"))
+            if targets:
+                return str(targets[0].parent.parent)
+        elif "update_dir" in params["type"]:
+            targets: List[Path] = list(
+                folder.rglob("**/Pack/ActorObserverByActorTagTag.sbactorpack")
+            )
+            if targets:
+                return str(targets[0].parent.parent.parent)
+        elif "dlc_dir" in params["type"]:
+            targets: List[Path] = list(folder.rglob("**/Pack/AocMainField.pack"))
+            if targets:
+                return str(targets[0].parent.parent)
+        return params["folder"]
+
     def parse_cemu_settings(self, params):
         try:
             cemu = Path(params["folder"])
@@ -115,6 +159,7 @@ class Api:
                 "game_dir": str(game_dir),
                 "update_dir": str(update_dir) if update_dir else "",
                 "dlc_dir": str(dlc_dir) if dlc_dir else "",
+                "export_dir": str(cemu / "graphicPacks" / "BreathOfTheWild_BCML"),
             }
         except Exception as err:  # pylint: disable=broad-except
             print(err)
@@ -123,8 +168,24 @@ class Api:
     def get_settings(self, params=None):
         return util.get_settings()
 
+    @win_or_lose
+    def make_shortcut(self, params):
+        util.create_shortcuts(params["desktop"], not params["desktop"])
+
+    def get_user_langs(self, params):
+        if params["dir"] and Path(params["dir"]).exists():
+            return list(util.get_user_languages(Path(params["dir"])))
+        else:
+            return []
+
     def save_settings(self, params):
         print("Saving settings, BCML will reload momentarily...")
+        if util.get_settings("wiiu") != params["settings"]["wiiu"]:
+            util.clear_all_caches()
+            if hasattr(self, "gb_api"):
+                self.gb_api.reset_update_time(params["settings"]["wiiu"])
+                del self.gb_api
+                self.gb_api = GameBananaDb()
         util.get_settings.settings = params["settings"]
         util.save_settings()
 
@@ -172,6 +233,22 @@ class Api:
         util.vprint(mods)
         return mods
 
+    def save_mod_list(self, params=None):
+        result = self.window.create_file_dialog(
+            webview.SAVE_DIALOG,
+            file_types=("JSON File (*.json)",),
+            allow_multiple=False,
+        )
+        if not result:
+            return
+        out = Path(result if isinstance(result, str) else result[0])
+        out.write_text(
+            json.dumps(
+                [mod.to_json() for mod in util.get_installed_mods(disabled=True)],
+                indent=4,
+            )
+        )
+
     def get_mod_info(self, params):
         mod = BcmlMod.from_json(params["mod"])
         util.vprint(mod)
@@ -200,23 +277,33 @@ class Api:
     def file_pick(self, params=None):
         if not params:
             params = {}
-        return (
-            self.window.create_file_dialog(
-                file_types=(
+        result = self.window.create_file_dialog(
+            file_types=params.get(
+                "types",
+                (
                     "All supported files (*.bnp;*.7z;*.zip;*.rar;*.txt;*.json)",
                     "Packaged mods (*.bnp;*.7z;*.zip;*.rar)",
                     "Mod meta (*.txt;*.json)",
                     "All files (*.*)",
-                )
-                if "types" not in params
-                else params["types"],
-                allow_multiple=True if "multiple" not in params else params["multiple"],
-            )
-            or []
+                ),
+            ),
+            allow_multiple=params.get("multiple", True),
         )
+        return result or []
+
+    def file_drop(self, params):
+        file = Path(tempfile.mkdtemp()) / params["file"]
+        file.write_bytes(base64.b64decode(params["data"]))
+        return str(file)
 
     def get_options(self):
-        opts = []
+        opts = [
+            {
+                "name": "general",
+                "friendly": "general options",
+                "options": {"base_priority": "Default to lowest priority"},
+            }
+        ]
         for merger in mergers.get_mergers():
             merger = merger()
             opts.append(
@@ -234,6 +321,42 @@ class Api:
             for b in [(b.stem.split("---"), str(b)) for b in install.get_backups()]
         ]
 
+    def get_profiles(self):
+        return [
+            {"name": (d / ".profile").read_text("utf-8"), "path": str(d)}
+            for d in {d for d in util.get_profiles_dir().glob("*") if d.is_dir()}
+        ]
+
+    def get_current_profile(self):
+        profile = util.get_modpack_dir() / ".profile"
+        if not (util.get_modpack_dir() / ".profile").exists():
+            profile.write_text("Default")
+            return "Default"
+        return profile.read_text("utf-8")
+
+    @win_or_lose
+    @install.refresher
+    def set_profile(self, params):
+        mod_dir = util.get_modpack_dir()
+        rmtree(mod_dir)
+        copytree(params["profile"], mod_dir)
+
+    @win_or_lose
+    def delete_profile(self, params):
+        rmtree(params["profile"])
+
+    @win_or_lose
+    def save_profile(self, params):
+        mod_dir = util.get_modpack_dir()
+        profile = mod_dir / ".profile"
+        profile.write_text(params["profile"])
+        profile_dir = util.get_profiles_dir() / util.get_safe_pathname(
+            params["profile"]
+        )
+        if profile_dir.exists():
+            rmtree(profile_dir)
+        copytree(mod_dir, profile_dir)
+
     def check_mod_options(self, params):
         metas = {
             mod: install.extract_mod_meta(Path(mod))
@@ -250,7 +373,7 @@ class Api:
     @install.refresher
     def install_mod(self, params: dict):
         util.vprint(params)
-        with Pool(maxtasksperchild=1000) as pool:
+        with Pool(maxtasksperchild=500) as pool:
             selects = (
                 params["selects"] if "selects" in params and params["selects"] else {}
             )
@@ -272,59 +395,45 @@ class Api:
                 pool.terminate()
                 raise
 
-    @win_or_lose
-    @install.refresher
     def update_mod(self, params):
         try:
             update_file = self.file_pick({"multiple": False})[0]
         except IndexError:
-            return
+            raise Exception("canceled")
         mod = BcmlMod.from_json(params["mod"])
         if (mod.path / "options.json").exists():
-            options = json.loads(
-                (mod.path / "options.json").read_text(), encoding="utf-8"
-            )
+            options = json.loads((mod.path / "options.json").read_text())
         else:
             options = {}
-        remergers = mergers.get_mergers_for_mod(mod)
         rmtree(mod.path)
-        with Pool(maxtasksperchild=1000) as pool:
+        with Pool(maxtasksperchild=500) as pool:
             new_mod = install.install_mod(
                 Path(update_file),
                 insert_priority=mod.priority,
                 options=options,
                 pool=pool,
+                updated=True,
             )
-            remergers |= {
-                m
-                for m in mergers.get_mergers_for_mod(new_mod)
-                if m.NAME not in {m.NAME for m in remergers}
-            }
-            try:
-                install.refresh_merges()
-            except Exception:  # pylint: disable=broad-except
-                pool.terminate()
-                raise
 
     @win_or_lose
-    @install.refresher
     def reprocess(self, params):
         mod = BcmlMod.from_json(params["mod"])
         rmtree(mod.path / "logs")
         if (mod.path / "options.json").exists():
-            options = json.loads(
-                (mod.path / "options.json").read_text(), encoding="utf-8"
-            )
+            options = json.loads((mod.path / "options.json").read_text())
         else:
             options = {}
         install.generate_logs(mod.path, options)
-        install.refresh_merges()
 
     @win_or_lose
     @install.refresher
     def uninstall_all(self):
         for folder in {d for d in util.get_modpack_dir().glob("*") if d.is_dir()}:
-            rmtree(folder)
+            rmtree(folder, onerror=install.force_del)
+        if not util.get_settings("no_cemu"):
+            shutil.rmtree(
+                util.get_cemu_dir() / "graphicPacks" / "bcmlPatches", ignore_errors=True
+            )
 
     @win_or_lose
     def apply_queue(self, params):
@@ -333,7 +442,7 @@ class Api:
             mod = BcmlMod.from_json(move_mod["mod"])
             mods.append(mod)
             mod.change_priority(move_mod["priority"])
-        with Pool(maxtasksperchild=1000) as pool:
+        with Pool(maxtasksperchild=500) as pool:
             for i in params["installs"]:
                 print(i)
                 mods.append(
@@ -344,12 +453,11 @@ class Api:
                         pool=pool,
                     )
                 )
-                print("Remerging...")
-                try:
-                    install.refresh_merges()
-                except Exception:  # pylint: disable=broad-except
-                    pool.terminate()
-                    raise
+            try:
+                install.refresh_merges()
+            except Exception:  # pylint: disable=broad-except
+                pool.terminate()
+                raise
         install.refresh_master_export()
 
     @win_or_lose
@@ -357,11 +465,11 @@ class Api:
         mod = BcmlMod.from_json(params["mod"])
         action = params["action"]
         if action == "enable":
-            install.enable_mod(mod)
+            install.enable_mod(mod, wait_merge=True)
         elif action == "disable":
-            install.disable_mod(mod)
+            install.disable_mod(mod, wait_merge=True)
         elif action == "uninstall":
-            install.uninstall_mod(mod)
+            install.uninstall_mod(mod, wait_merge=True)
         elif action == "update":
             self.update_mod(params)
         elif action == "reprocess":
@@ -477,7 +585,7 @@ class Api:
         except IndexError:
             return
         tmp_dir = Path(mkdtemp())
-        x_args = [get_7z_path(), "x", str(file), f"-o{str(tmp_dir)}"]
+        x_args = [get_7z_path(), "x", str(file), f"-o{str(tmp_dir)}", "-aoa"]
         if system() == "Windows":
             run(
                 x_args,
@@ -498,7 +606,7 @@ class Api:
         if not util.get_installed_mods():
             raise Exception("No mods installed to export.")
         out = self.window.create_file_dialog(
-            webviewb.SAVE_DIALOG,
+            webview.SAVE_DIALOG,
             file_types=(
                 f"{('Graphic Pack' if util.get_settings('wiiu') else 'Atmosphere')} (*.zip)",
                 "BOTW Nano Patch (*.bnp)",
@@ -518,7 +626,7 @@ class Api:
     @win_or_lose
     def create_bnp(self, params):
         out = self.window.create_file_dialog(
-            webviewb.SAVE_DIALOG,
+            webview.SAVE_DIALOG,
             file_types=("BOTW Nano Patch (*.bnp)", "All files (*.*)"),
             save_filename=util.get_safe_pathname(params["name"]) + ".bnp",
         )
@@ -526,8 +634,9 @@ class Api:
             raise Exception("canceled")
         meta = params.copy()
         del meta["folder"]
-        meta["options"] = params["selects"]
-        del meta["selects"]
+        meta["options"] = params.get("selects", {})
+        if "selects" in meta:
+            del meta["selects"]
         dev.create_bnp_mod(
             mod=Path(params["folder"]),
             output=Path(out if isinstance(out, str) else out[0]),
@@ -549,7 +658,7 @@ class Api:
         mod = install.open_mod(bnp)
         warnings = dev.convert_mod(mod, params["wiiu"], params["warn"])
         out = self.window.create_file_dialog(
-            webviewb.SAVE_DIALOG,
+            webview.SAVE_DIALOG,
             file_types=("BOTW Nano Patch (*.bnp)", "All files (*.*)"),
             save_filename=bnp.stem + f"_{'wiiu' if params['wiiu'] else 'switch'}.bnp",
         )
@@ -581,7 +690,7 @@ class Api:
     def get_existing_meta(self, params):
         path = Path(params["path"])
         if (path / "info.json").exists():
-            return json.loads((path / "info.json").read_text("utf-8"), encoding="utf-8")
+            return json.loads((path / "info.json").read_text("utf-8"))
         if (path / "rules.txt").exists():
             return upgrade.parse_rules(path / "rules.txt")
         return {}
@@ -613,7 +722,14 @@ class Api:
                     )
                 )
             install.install_mod(
-                mod, merge_now=True,
+                mod,
+                merge_now=True,
+                options={
+                    "options": {},
+                    "disable": [
+                        m.NAME for m in mergers.get_mergers() if m.NAME != "rstb"
+                    ],
+                },
             )
             (mod / util.get_content_path() / "System" / "Resource").mkdir(
                 parents=True, exist_ok=True
@@ -644,7 +760,7 @@ class Api:
                 options={"options": {"texts": {"all_langs": True}}, "disable": []},
             )
             out = self.window.create_file_dialog(
-                webviewb.SAVE_DIALOG,
+                webview.SAVE_DIALOG,
                 file_types=(
                     f"{('Graphic Pack' if util.get_settings('wiiu') else 'Atmosphere')} (*.zip)",
                     "BOTW Nano Patch (*.bnp)",
@@ -653,7 +769,7 @@ class Api:
             )
             if out:
                 output = Path(out if isinstance(out, str) else out[0])
-                install.export(output)
+                install.export(output, standalone=True)
 
     def get_mod_edits(self, params=None):
         mod = BcmlMod.from_json(params["mod"])
@@ -675,7 +791,7 @@ class Api:
             return
         tmp_dir = install.open_mod(path)
         output = self.window.create_file_dialog(
-            webviewb.SAVE_DIALOG, file_types=tuple(["BOTW Nano Patch (*.bnp)"])
+            webview.SAVE_DIALOG, file_types=tuple(["BOTW Nano Patch (*.bnp)"])
         )
         if not output:
             return
@@ -699,11 +815,8 @@ class Api:
 
     @win_or_lose
     def update_bcml(self):
-        if util.get_latest_bcml() <= VERSION:
-            return
-        exe = util.get_python_exe().replace("pythonw", "python")
+        exe = str(util.get_python_exe(False))
         args = [
-            exe,
             "-m",
             "pip",
             "install",
@@ -714,46 +827,56 @@ class Api:
         ]
         if DEBUG:
             args.insert(-2, "--pre")
-        parent = util.get_exec_dir().parent
-        if parent.name == "pkgs":
-            try:
-                (parent / "bcml").rename(parent / "bcml.bak")
-            except (OSError, PermissionError, FileExistsError, FileNotFoundError):
-                pass
         if SYSTEM == "Windows":
-            result = run(
-                args,
-                creationflags=util.CREATE_NO_WINDOW,
-                capture_output=True,
-                check=False,
-                text=True,
-            )
+            with NamedTemporaryFile("w", suffix=".bat", delete=False) as updater:
+                updater.write(
+                    "@echo off\n"
+                    'taskkill /fi "WINDOWTITLE eq BOTW Cross-Platform Mod Loader"\n'
+                    f"\"{exe}\" {' '.join(args)}\n"
+                    "echo Finished updating, will launch BCML in a moment!\n"
+                    "timeout 2 >nul 2>&1\n"
+                    f'start "" "{str(util.get_python_exe(True))}" -m bcml\n'
+                )
+                file = updater.name
+            os.system(f"timeout 2 >nul 2>&1 && start cmd /c {file}")
         else:
-            result = run(args, capture_output=True, check=False, text=True,)
-        if result.stderr:
-            if parent.name == "pkgs":
-                try:
-                    (parent / "bcml.bak").rename(parent / "bcml")
-                except (OSError, PermissionError, FileExistsError, FileNotFoundError):
-                    pass
-            raise RuntimeError(result.stderr)
+            with NamedTemporaryFile("w", suffix=".sh", delete=False) as updater:
+                updater.write(
+                    "#!/usr/bin/bash\n"
+                    "sleep 2\n"
+                    f"\"{exe}\" {' '.join(args)}\n"
+                    "echo Finished updating, will launch BCML in a moment!\n"
+                    f"{exe} -m bcml"
+                )
+                file = updater.name
+            Popen(["/bin/sh", file], start_new_session=True)
+        for win in webview.windows:
+            win.destroy()
 
     def restart(self):
         opener = Thread(target=start_new_instance)
         opener.start()
-        for win in webviewb.windows:
+        for win in webview.windows:
             win.destroy()
 
     def is_wiiu(self):
         return util.get_settings("wiiu")
 
+    @win_or_lose
     def init_gb(self):
         self.gb_api = GameBananaDb()
+        if util.get_settings("auto_gb"):
+            self.gb_api.update_db()
+
+    @win_or_lose
+    def update_gb(self):
+        self.gb_api.update_db()
 
     def get_gb_pages(self, category: str = None, search: str = None) -> int:
         mods = self.gb_api.mods
         if search:
             mods = self.gb_api.search(search)
+            mods = mods[0] + mods[1]
         return ceil(
             len(
                 mods if not category else [m for m in mods if m["category"] == category]
@@ -765,8 +888,6 @@ class Api:
         self, page: int, sort: str = "new", category: str = None, search: str = None
     ):
         mods = self.gb_api.mods
-        if search:
-            mods = self.gb_api.search(search)
         key = {
             "new": itemgetter("updated"),
             "old": itemgetter("updated"),
@@ -774,14 +895,46 @@ class Api:
             "abc": itemgetter("name"),
             "likes": itemgetter("likes"),
         }.get(sort)
-        mods = sorted(
-            mods if not category else [m for m in mods if m["category"] == category],
-            key=key,
-            reverse=sort not in {"old", "abc"},
-        )
+        if not search:
+            mods = sorted(
+                mods
+                if not category
+                else [m for m in mods if m["category"] == category],
+                key=key,
+                reverse=sort not in {"old", "abc"},
+            )
+        else:
+            title_matches, desc_matches = self.gb_api.search(search)
+            exacts = [
+                title_matches.pop(i)
+                for i in range(len(title_matches) - 1)
+                if title_matches[i]["name"].lower() == search.lower()
+            ]
+            mods = (
+                exacts
+                + sorted(
+                    title_matches
+                    if not category
+                    else [m for m in title_matches if m["category"] == category],
+                    key=key,
+                    reverse=sort not in {"old", "abc"},
+                )
+                + sorted(
+                    desc_matches
+                    if not category
+                    else [m for m in desc_matches if m["category"] == category],
+                    key=key,
+                    reverse=sort not in {"old", "abc"},
+                )
+            )
         last = min(page * 24, len(mods))
         return mods[(page - 1) * 24 : last]
 
+    @win_or_lose
+    def update_gb_mod(self, mod_id: str):
+        self.gb_api.update_mod(str(mod_id))
+        return self.gb_api.get_mod_by_id(str(mod_id))
+
     @win_or_lose
     def install_gb_mod(self, file: dict):
         path = Path(mkdtemp()) / file["_sFile"]
