@@ -8,6 +8,7 @@ from functools import partial
 from multiprocessing import Pool
 from pathlib import Path
 from typing import Dict, Union, List, Tuple
+from zlib import crc32
 
 import oead
 from oead.byml import Hash, Array  # pylint: disable=import-error
@@ -173,9 +174,7 @@ def get_modded_map(map_unit: Union[Map, tuple], tmp_dir: Path) -> Hash:
     return oead.byml.from_binary(map_bytes)
 
 
-def get_map_diff(
-    map_unit: Map, tmp_dir: Path, no_del: bool = False, link_del: bool = False
-) -> Hash:
+def get_map_diff(map_unit: Map, tmp_dir: Path, new_hashes: bool = False) -> Hash:
     mod_map = get_modded_map(map_unit, tmp_dir)
     stock_map = True
     for obj in mod_map["Objs"]:
@@ -187,21 +186,11 @@ def get_map_diff(
 
     def diff_objs() -> Hash:
         base_hashes = [int(obj["HashId"]) for obj in base_map["Objs"]]
-        base_links = (
-            set()
-            if link_del
-            else {
-                int(link["DestUnitHashId"])
-                for obj in base_map["Objs"]
-                for link in obj.get("LinksToObj", Array())
-                if "LinksToObj" in obj
-            }
-        )
         mod_hashes = [int(obj["HashId"]) for obj in mod_map["Objs"]]
 
         diffs = Hash()
         diffs["add"] = Array(
-            {obj for obj in mod_map["Objs"] if int(obj["HashId"]) not in base_hashes}
+            [obj for obj in mod_map["Objs"] if int(obj["HashId"]) not in base_hashes]
         )
         diffs["mod"] = Hash(
             {
@@ -212,14 +201,27 @@ def get_map_diff(
             }
         )
         diffs["del"] = Array(
-            {
-                oead.U32(hash_id)
-                for hash_id in base_hashes
-                if hash_id not in {*mod_hashes, *base_links}
-            }
-            if not no_del
-            else set()
+            [
+                oead.U32(h)
+                for h in {
+                    hash_id for hash_id in base_hashes if hash_id not in mod_hashes
+                }
+            ]
         )
+
+        if new_hashes:
+            hash_map: Dict[int, int] = {}
+            for obj in diffs["add"]:
+                new_hash = crc32(oead.byml.to_text(obj).encode("utf8"))
+                hash_map[obj["HashId"].v] = new_hash
+                obj["HashId"] = oead.U32(new_hash)
+            for obj in [*diffs["add"], *[v for k, v in diffs["mod"].items()]]:
+                if "LinksToObj" in obj:
+                    for link in obj["LinksToObj"]:
+                        if link["DestUnitHashId"].v in hash_map:
+                            link["DestUnitHashId"] = oead.U32(
+                                hash_map[link["DestUnitHashId"].v]
+                            )
         return diffs
 
     def diff_rails() -> Hash:
@@ -228,11 +230,11 @@ def get_map_diff(
 
         diffs = Hash()
         diffs["add"] = Array(
-            {
+            [
                 rail
                 for rail in mod_map["Rails"]
                 if int(rail["HashId"]) not in base_hashes
-            }
+            ]
         )
         diffs["mod"] = Hash(
             {
@@ -243,9 +245,12 @@ def get_map_diff(
             }
         )
         diffs["del"] = Array(
-            {oead.U32(hash_id) for hash_id in base_hashes if hash_id not in mod_hashes}
-            if not no_del
-            else set()
+            [
+                oead.U32(h)
+                for h in {
+                    hash_id for hash_id in base_hashes if hash_id not in mod_hashes
+                }
+            ]
         )
         return diffs
 
@@ -265,19 +270,16 @@ def get_map_diff(
 def generate_modded_map_log(
     tmp_dir: Path,
     modded_mubins: List[Path],
-    no_del: bool = False,
-    link_del: bool = False,
     pool: multiprocessing.pool.Pool = None,
+    new_hashes: bool = False,
 ) -> Hash:
     modded_maps = consolidate_map_files(modded_mubins)
-    this_pool = pool or Pool()
+    this_pool = pool or Pool(maxtasksperchild=500)
     diffs = oead.byml.Hash(
         {
             map_unit: oead.byml.from_text(diff)
             for map_unit, diff in this_pool.imap_unordered(
-                partial(
-                    get_map_diff, tmp_dir=tmp_dir, no_del=no_del, link_del=link_del
-                ),
+                partial(get_map_diff, tmp_dir=tmp_dir, new_hashes=new_hashes),
                 modded_maps,
             )
         }
@@ -289,7 +291,7 @@ def generate_modded_map_log(
 
 
 def merge_map(
-    map_pair: tuple, rstb_calc: rstb.SizeCalculator, no_del: bool = False
+    map_pair: tuple, rstb_calc: rstb.SizeCalculator
 ) -> Dict[str, Tuple[str, int]]:
     map_unit, changes = map_pair[0], map_pair[1]
     util.vprint(f'Merging {len(changes)} versions of {"_".join(map_unit)}...')
@@ -300,31 +302,30 @@ def merge_map(
             new_map["Objs"][stock_obj_hashes.index(int(hash_id))] = actor
         except ValueError:
             changes["Objs"]["add"].append(actor)
-    if not no_del:
-        for map_del in sorted(
-            changes["Objs"]["del"],
-            key=lambda change: stock_obj_hashes.index(int(change))
-            if int(change) in stock_obj_hashes
-            else -1,
-            reverse=True,
-        ):
-            if int(map_del) in stock_obj_hashes:
+    for map_del in sorted(
+        changes["Objs"]["del"],
+        key=lambda change: stock_obj_hashes.index(change)
+        if change in stock_obj_hashes
+        else -1,
+        reverse=True,
+    ):
+        if int(map_del) in stock_obj_hashes:
+            try:
+                new_map["Objs"].pop(stock_obj_hashes.index(map_del))
+            except IndexError:
                 try:
-                    new_map["Objs"].pop(stock_obj_hashes.index(int(map_del)))
-                except IndexError:
-                    try:
-                        obj_to_delete = next(
-                            iter(
-                                [
-                                    actor
-                                    for actor in new_map["Objs"]
-                                    if actor["HashId"] == map_del
-                                ]
-                            )
+                    obj_to_delete = next(
+                        iter(
+                            [
+                                actor
+                                for actor in new_map["Objs"]
+                                if actor["HashId"] == map_del
+                            ]
                         )
-                        new_map["Objs"].remove(obj_to_delete)
-                    except (StopIteration, ValueError):
-                        util.vprint(f"Could not delete actor with HashId {map_del}")
+                    )
+                    new_map["Objs"].remove(obj_to_delete)
+                except (StopIteration, ValueError):
+                    util.vprint(f"Could not delete actor with HashId {map_del}")
     new_map["Objs"].extend(
         [
             change
@@ -341,31 +342,30 @@ def merge_map(
                 new_map["Rails"][stock_rail_hashes.index(int(hash_id))] = rail
             except ValueError:
                 changes["Rails"]["add"].append(rail)
-        if not no_del:
-            for map_del in sorted(
-                changes["Rails"]["del"],
-                key=lambda change: stock_rail_hashes.index(int(change))
-                if int(change) in stock_rail_hashes
-                else -1,
-                reverse=True,
-            ):
-                if int(map_del) in stock_rail_hashes:
+        for map_del in sorted(
+            changes["Rails"]["del"],
+            key=lambda change: stock_rail_hashes.index(int(change))
+            if int(change) in stock_rail_hashes
+            else -1,
+            reverse=True,
+        ):
+            if int(map_del) in stock_rail_hashes:
+                try:
+                    new_map["Rails"].pop(stock_rail_hashes.index(int(map_del)))
+                except IndexError:
                     try:
-                        new_map["Rails"].pop(stock_rail_hashes.index(int(map_del)))
-                    except IndexError:
-                        try:
-                            obj_to_delete = next(
-                                iter(
-                                    [
-                                        rail
-                                        for rail in new_map["Rails"]
-                                        if rail["HashId"] == map_del
-                                    ]
-                                )
+                        obj_to_delete = next(
+                            iter(
+                                [
+                                    rail
+                                    for rail in new_map["Rails"]
+                                    if rail["HashId"] == map_del
+                                ]
                             )
-                            new_map["Rails"].remove(obj_to_delete)
-                        except (StopIteration, ValueError):
-                            util.vprint(f"Could not delete rail with HashId {map_del}")
+                        )
+                        new_map["Rails"].remove(obj_to_delete)
+                    except (StopIteration, ValueError):
+                        util.vprint(f"Could not delete rail with HashId {map_del}")
         new_map["Rails"].extend(
             [
                 change
@@ -428,14 +428,19 @@ def merge_map(
     }
 
 
-def get_dungeonstatic_diff(file: Path) -> dict:
-    base_pos = oead.byml.from_binary(
-        util.decompress(
-            (util.get_aoc_dir() / "Map" / "CDungeon" / "Static.smubin").read_bytes()
-        )
-    )["StartPos"]
-
-    mod_pos = oead.byml.from_binary(util.decompress(file.read_bytes()))["StartPos"]
+def get_dungeonstatic_diff(mod_pos: Array) -> dict:
+    try:
+        base_pos = oead.byml.from_binary(
+            util.decompress(
+                (util.get_aoc_dir() / "Map" / "CDungeon" / "Static.smubin").read_bytes()
+            )
+        )["StartPos"]
+    except FileNotFoundError:
+        base_pos = oead.byml.from_binary(
+            util.get_nested_file_bytes(
+                f"{util.get_game_file('Pack/Bootup.pack')}//Map/CDungeon/Static.smubin"
+            )
+        )["StartPos"]
 
     base_dungeons = [str(dungeon["Map"]) for dungeon in base_pos]
     diffs = {}
@@ -457,13 +462,24 @@ def get_dungeonstatic_diff(file: Path) -> dict:
 def merge_dungeonstatic(diffs: dict = None):
     """Merges all changes to the CDungeon Static.smubin"""
     if not diffs:
+        try:
+            (util.get_master_modpack_dir() / "logs" / "dungeonstatic.smubin").unlink()
+        except:
+            pass
         return
 
-    new_static = oead.byml.from_binary(
-        util.decompress(
-            (util.get_aoc_dir() / "Map" / "CDungeon" / "Static.smubin").read_bytes()
+    try:
+        new_static = oead.byml.from_binary(
+            util.decompress(
+                (util.get_aoc_dir() / "Map" / "CDungeon" / "Static.smubin").read_bytes()
+            )
+        )
+    except FileNotFoundError:
+        new_static = oead.byml.from_binary(
+            util.get_nested_file_bytes(
+                f"{util.get_game_file('Pack/Bootup.pack')}//Map/CDungeon/Static.smubin"
+            )
         )
-    )
 
     base_dungeons = [str(dungeon["Map"]) for dungeon in new_static["StartPos"]]
     for dungeon, diff in diffs.items():
@@ -473,20 +489,29 @@ def merge_dungeonstatic(diffs: dict = None):
             for key, value in diff.items():
                 new_static["StartPos"][base_dungeons.index(dungeon)][key] = value
 
-    output_static = (
-        util.get_master_modpack_dir()
-        / util.get_dlc_path()
-        / ("0010" if util.get_settings("wiiu") else "")
-        / "Map"
-        / "CDungeon"
-        / "Static.smubin"
+    data = util.compress(
+        oead.byml.to_binary(new_static, big_endian=util.get_settings("wiiu"))
     )
-    output_static.parent.mkdir(parents=True, exist_ok=True)
-    output_static.write_bytes(
-        util.compress(
-            oead.byml.to_binary(new_static, big_endian=util.get_settings("wiiu"))
+    try:
+        util.get_aoc_dir()
+        output_static = (
+            util.get_master_modpack_dir()
+            / util.get_dlc_path()
+            / ("0010" if util.get_settings("wiiu") else "")
+            / "Map"
+            / "CDungeon"
+            / "Static.smubin"
         )
-    )
+    except FileNotFoundError:
+        output_static = util.get_master_modpack_dir() / "logs" / "dungeonstatic.smubin"
+        util.inject_file_into_sarc(
+            "Map/CDungeon/Static.smubin",
+            data,
+            "Pack/Bootup.pack",
+            create_sarc=True,
+        )
+    output_static.parent.mkdir(parents=True, exist_ok=True)
+    output_static.write_bytes(data)
 
 
 def parse_legacy_diff(text: str) -> Hash:
@@ -522,9 +547,8 @@ class MapMerger(mergers.Merger):
             return generate_modded_map_log(
                 mod_dir,
                 modded_mubins,
-                no_del=self._options.get("no_del", False),
-                link_del=self._options.get("link_del", False),
                 pool=self._pool,
+                new_hashes=self._options.get("new_hashes", False),
             )
         return {}
 
@@ -579,7 +603,7 @@ class MapMerger(mergers.Merger):
                     "del": list(
                         set(
                             [
-                                hash_id
+                                hash_id.v
                                 for hashes in [mod["Objs"]["del"] for mod in mods]
                                 for hash_id in hashes
                             ]
@@ -592,7 +616,7 @@ class MapMerger(mergers.Merger):
                     "del": list(
                         set(
                             [
-                                hash_id
+                                hash_id.v
                                 for hashes in [
                                     mod["Rails"]["del"]
                                     for mod in mods
@@ -626,18 +650,6 @@ class MapMerger(mergers.Merger):
 
     @util.timed
     def perform_merge(self):
-        no_del = self._options.get("no_del", False)
-        aoc_pack = (
-            util.get_master_modpack_dir()
-            / util.get_dlc_path()
-            / ("0010" if util.get_settings("wiiu") else "")
-            / "Pack"
-            / "AocMainField.pack"
-        )
-        if not aoc_pack.exists() or aoc_pack.stat().st_size > 0:
-            print("Emptying AocMainField.pack...")
-            aoc_pack.parent.mkdir(parents=True, exist_ok=True)
-            aoc_pack.write_bytes(b"")
         shutil.rmtree(
             str(
                 util.get_master_modpack_dir()
@@ -667,14 +679,26 @@ class MapMerger(mergers.Merger):
         if not map_diffs:
             print("No map merge necessary")
             return
+        aoc_pack = (
+            util.get_master_modpack_dir()
+            / util.get_dlc_path()
+            / ("0010" if util.get_settings("wiiu") else "")
+            / "Pack"
+            / "AocMainField.pack"
+        )
+        if not aoc_pack.exists() or aoc_pack.stat().st_size > 0:
+            print("Emptying AocMainField.pack...")
+            aoc_pack.parent.mkdir(parents=True, exist_ok=True)
+            aoc_pack.write_bytes(b"")
 
         rstb_vals = {}
         rstb_calc = rstb.SizeCalculator()
         print("Merging modded map units...")
 
-        pool = self._pool or Pool()
+        pool = self._pool or Pool(maxtasksperchild=500)
         rstb_results = pool.map(
-            partial(merge_map, rstb_calc=rstb_calc, no_del=no_del), map_diffs.items(),
+            partial(merge_map, rstb_calc=rstb_calc),
+            map_diffs.items(),
         )
         for result in rstb_results:
             rstb_vals[result[util.get_dlc_path()][0]] = result[util.get_dlc_path()][1]
@@ -683,6 +707,25 @@ class MapMerger(mergers.Merger):
             pool.close()
             pool.join()
 
+        stock_static = [m for m in map_diffs if m[1] == "Static"]
+        if stock_static:
+            title_path = (
+                util.get_master_modpack_dir()
+                / util.get_content_path()
+                / "Pack"
+                / "TitleBG.pack"
+            )
+            if not title_path.exists():
+                title_path.parent.mkdir(parents=True, exist_ok=True)
+                shutil.copyfile(util.get_game_file("Pack/TitleBG.pack"), title_path)
+            title_bg: oead.SarcWriter = oead.SarcWriter.from_sarc(
+                oead.Sarc(title_path.read_bytes())
+            )
+            for static in stock_static:
+                del title_bg.files[
+                    f"Map/MainField/{static[0]}/{static[0]}_Static.smubin"
+                ]
+            title_path.write_bytes(title_bg.write()[1])
         print("Adjusting RSTB...")
         log_path.parent.mkdir(parents=True, exist_ok=True)
         with log_path.open("w", encoding="utf-8") as l_file:
@@ -692,8 +735,7 @@ class MapMerger(mergers.Merger):
 
     def get_checkbox_options(self):
         return [
-            ("no_del", "Never remove stock actors from merged maps"),
-            ("link_del", "Allow deleting actors with links from merged maps"),
+            ("new_hashes", "Generate unique hashes for added map actors"),
         ]
 
     def get_mod_edit_info(self, mod: util.BcmlMod) -> set:
@@ -726,7 +768,22 @@ class DungeonStaticMerger(mergers.Merger):
         )
         if dstatic_path.exists():
             print("Logging changes to shrine entry coordinates...")
-            return get_dungeonstatic_diff(dstatic_path)
+            return get_dungeonstatic_diff(
+                oead.byml.from_binary(util.decompress(dstatic_path.read_bytes()))[
+                    "StartPos"
+                ]
+            )
+        elif (
+            f"{util.get_content_path()}/Pack/Bootup.pack//Map/CDungeon/Static.smubin"
+            in modded_files
+        ):
+            return get_dungeonstatic_diff(
+                oead.byml.from_binary(
+                    util.get_nested_file_bytes(
+                        f"{mod_dir}/{util.get_content_path()}/Pack/Bootup.pack//Map/CDungeon/Static.smubin"
+                    )
+                )["StartPos"]
+            )
         else:
             return {}
 
@@ -782,5 +839,22 @@ class DungeonStaticMerger(mergers.Merger):
     def get_checkbox_options(self):
         return []
 
+    @staticmethod
+    def is_bootup_injector():
+        try:
+            util.get_aoc_dir()
+            return False
+        except FileNotFoundError:
+            return True
+
+    def get_bootup_injection(self):
+        tmp_sarc = util.get_master_modpack_dir() / "logs" / "dungeonstatic.smubin"
+        if tmp_sarc.exists():
+            return (
+                "Map/CDungeon/Static.smubin",
+                tmp_sarc,
+            )
+        return
+
     def get_mod_edit_info(self, mod: util.BcmlMod) -> set:
         return set(self.get_mod_diff(mod).keys())
